import unittest

from nymms.scheduler.lock.SchedulerLock import SchedulerLock

NOW = 1391115581.238759
DURATION = 30

class TestSchedulerLock(unittest.TestCase):
    def setUp(self):
        self.lock = SchedulerLock(DURATION)

    def test_lock_expired(self):
        no_lock = None
        self.assertIs(self.lock.lock_expired(no_lock, NOW), True)
        expired_lock = NOW - (DURATION + 5)
        self.assertIs(self.lock.lock_expired(expired_lock, NOW),
                          True)
        valid_lock = NOW + 5
        self.assertIs(self.lock.lock_expired(valid_lock, NOW), False)



import time
import re
import logging
import uuid

logger = logging.getLogger(__name__)


class ReactorSuppress(object):
    """Record of the suppression to store in SDB/etc"""
    def __init__(self, item):
        self.comment = str(item['comment'])
        self.expires = int(item['expires'])
        self.ipaddr = str(item['ipaddr'])
        self.regex = str(item['regex'])
        self.re = re.compile(self.regex)
        self.userid = str(item['userid'])
        self.rowkey = str(item.get('rowkey', uuid.uuid4()))
        self.active = str(item.get('active', True))
        self.created_at = int(item.get('created_at', time.time()))

    def dict(self):
        return {
                'active': self.active,
                'comment': self.comment,
                'created_at': self.created_at,
                'expires': self.expires,
                'ipaddr': self.ipaddr,
                'regex': self.regex,
                'userid': self.userid,
                'rowkey': self.rowkey
                }


class SuppressFilterBackend(object):
    """Parent SuppressFilterBackend class.  Don't use this directly!

    You need to define:
    add_suppression(self, suppress)
    get_suppressions(self, expire, active)
    deactivate_suppression(self, rowkey)
    """
    def __init__(self, timeout):
        self._cache_timeout = timeout
        self._cache_expire_time = 0
        self._cached_suppressions = []

    def get_active_suppressions(self):
        """Returns a list of suppression filters which are currently
        active in SDB"""
        now = int(time.time())
        return self.get_suppressions(now, True)

    def get_cached_current_suppressions(self):
        """Returns a list of currently active suppression filters"""
        now = int(time.time())

        if self._cache_expire_time < now:
            logger.debug("Refreshing reactor suppression cache")
            self._cache_expire_time = now + self._cache_timeout
            self._cached_suppressions = []
            filters = self.get_active_suppressions()
            for item in filters:
                self._cached_suppressions.append(item)

        return self._cached_suppressions

    def is_suppressed(self, message):
        """Returns True if given message matches one of our active filters"""
        suppressions = self.get_cached_current_suppressions()
        for item in suppressions:
            if item.re.search(message):
                return item
        return False

    def add_suppression(self, suppress):
        raise NotImplementedError

    def get_suppressions(self, **kwargs):
        raise NotImplementedError

    def deactivate_suppression(self, **kwargs):
        raise NotImplementedError

    def deactivate_all_suppressions(self):
        """Deactivates all the active suppression filters we have currently."""
        deactivated = []
        for item in self.get_active_suppressions():
            logger.debug("Deactivating %s", item.rowkey)
            deactivated.append(item.rowkey)
            self.deactivate_suppression(item.rowkey)
        return deactivated



import logging
import copy
import os
import collections

logger = logging.getLogger(__name__)

validator = None
try:
    import validictory
    validator = validictory.validate
except ImportError:
    logger.warning("Unable to import validictory - skipping config "
                   "validation.")

from nymms.config import yaml_config
from nymms.utils import deep_update
from nymms import exceptions


default_conf_dir = '/etc/nymms'

SCHEMA = {
    'type': 'object',
    'properties': {
        'monitor_timeout': {
            'type': 'integer', 'minimum': 0,
        },
        'resources': {
            'type': 'string',
        },
        'region': {
            'type': 'string',
        },
        'state_domain': {
            'type': 'string',
        },
        'tasks_queue': {
            'type': 'string',
        },
        'results_topic': {
            'type': 'string',
        },
        'private_context_file': {
            'type': 'string',
        },
        'task_expiration': {
            'type': 'integer', 'minimum': 0,
        },
        'suppress': {
            'type': 'object',
            'properties': {
                'domain': {
                    'type': 'string',
                },
                'cache_timeout': {
                    'type': 'integer', 'minimum': 0,
                },
            }
        },
        'probe': {
            'type': 'object',
            'properties': {
                'max_retries': {
                    'type': 'integer', 'minimum': 0,
                },
                'queue_wait_time': {
                    'type': 'integer', 'minimum': 0, 'maximum': 20,
                },
                'retry_delay': {
                    'type': 'integer', 'minimum': 0,
                },
            }
        },
        'reactor': {
            'type': 'object',
            'properties': {
                'handler_config_path': {
                    'type': 'string',
                },
                'queue_name': {
                    'type': 'string',
                },
                'queue_wait_time': {
                    'type': 'integer', 'minimum': 0, 'maximum': 20,
                },
                'visibility_timeout': {
                    'type': 'integer', 'minimum': 5,
                },
            },
        },
        'scheduler': {
            'type': 'object',
            'properties': {
                'interval': {
                    'type': 'integer', 'minimum': 30,
                },
                'backend': {
                    'type': 'string',
                },
                'backend_args': {
                    'type': 'object',
                },
                'lock_backend': {
                    'type': 'string',
                },
                'lock_args': {
                    'type': 'object',
                },
            },
        },
    }
}

DEFAULTS = {
    'monitor_timeout': 15,
    'resources': os.path.join(default_conf_dir, 'resources.yaml'),
    'region': 'us-east-1',
    'state_domain': 'nymms_state',
    'tasks_queue': 'nymms_tasks',
    'results_topic': 'nymms_results',
    'private_context_file': os.path.join(default_conf_dir, 'private.yaml'),
    'task_expiration': 600,

    'probe': {
        'max_retries': 2,
        'queue_wait_time': 20,
        'retry_delay': 30,
    },

    'reactor': {
        'handler_config_path': os.path.join(default_conf_dir, 'handlers'),
        'queue_name': 'reactor_queue',
        'queue_wait_time': 20,
        'visibility_timeout': 30,
    },

    'scheduler': {
        'interval': 300,
        'backend': 'nymms.scheduler.backends.yaml_backend.YamlBackend',
        'backend_args': {
            'path': os.path.join(default_conf_dir, 'nodes.yaml'),
        },
        'lock_backend': 'SDB',
        'lock_args': {
            'duration': 360,
            'domain_name': 'nymms_locks',
            'lock_name': 'scheduler_lock',
        },
    },

    'suppress': {
        'domain': 'nymms_suppress',
        'cache_timeout': 60,
    }
}

settings = None
version = None


def load_config(path, force=False):
    global settings, version, DEFAULTS
    if settings and not force:
        return
    settings = copy.deepcopy(DEFAULTS)
    version, _config_settings = yaml_config.load_config(path)
    if _config_settings:
        deep_update(settings, _config_settings)
    if validator:
        try:
            validator(settings, SCHEMA)
        except ValueError as e:
            raise exceptions.InvalidConfig(path, e.message)
    logger.debug("Config loaded from '%s' with version '%s'." % (path,
                                                                 version))



import logging
import glob
import os
import sys
import time

import nymms
from nymms.daemon import NymmsDaemon
from nymms.config import yaml_config
from nymms.utils import load_object_from_string, logutil
from nymms.exceptions import OutOfDateState

logger = logging.getLogger(__name__)


class Reactor(NymmsDaemon):
    def __init__(self):
        self._handlers = {}
        self._suppress_backend = None
        super(Reactor, self).__init__()

    def _list_handler_configs(self, path):
        path = os.path.expanduser(path)
        return glob.glob(os.path.join(path, '*.conf'))

    def _get_handler_name(self, filename):
        return os.path.basename(filename)[:-5]

    def _load_handler(self, handler_name, config, **kwargs):
        enabled = config.pop('enabled', False)

        if not enabled:
            logger.debug("Handler %s 'enabled' is not set to true. "
                         "Skipping.", handler_name)
            return None
        cls_string = config.pop('handler_class')
        logger.debug('Initializing handler %s.', handler_name)
        try:
            handler_cls = load_object_from_string(cls_string)
            return handler_cls(config)
        except Exception as e:
            logutil.log_exception("Skipping handler %s due to "
                "unhandled exception:", handler_name, logger)
            return None

    def _load_handlers(self, handler_config_path, **kwargs):
        conf_files = self._list_handler_configs(handler_config_path)
        logger.debug("Loading handlers from %s", handler_config_path)
        for f in conf_files:
            handler_name = self._get_handler_name(f)
            # We could eventually have the handlers get loaded everytime and
            # update them if their config has changed (via config_version
            # below).  For now lets not get that tricky.
            if handler_name in self._handlers:
                if not kwargs.get('force_load_handlers', False):
                    logger.debug("Handler %s already loaded, skipping.",
                                 handler_name)
                    continue
            conf_version, config = yaml_config.load_config(f)
            handler = self._load_handler(handler_name, config, **kwargs)
            if handler:
                self._handlers[handler_name] = handler

        if not self._handlers:
            logger.error("No handlers loaded.  Exiting.")
            sys.exit(1)

    def get_result(self, **kwargs):
        raise NotImplementedError

    # TODO: This calls on _state_backend but setting up of the _state_backend
    #       needs to be handled in the subclass.  Not sure how I should handle
    #       this, but I really like the idea of these being base class
    #       methods since in reality all reactors should have some sort of
    #       state backend, even if its a no-op
    def get_state(self, task_id):
        return self._state_backend.get_state(task_id)

    def save_state(self, task_id, result, previous):
        return self._state_backend.save_state(task_id, result, previous)

    def is_suppressed(self, result):
        """Returns True if we should suppress the given result for event"""
        if not self._suppress_backend:
            logger.debug("is_suppressed(): No suppress backend, so returning False")
            return False
        suppression_filter = self._suppress_backend.is_suppressed(result.id)
        if suppression_filter:
            created_at = time.gmtime(suppression_filter.created_at)
            timestr = time.strftime("%Y-%m-%d %H:%M:%S UTC", created_at)
            logger.debug("Suppressed %s with '%s' (%s) created at %s",
                result.id,
                suppression_filter.regex,
                suppression_filter.rowkey,
                timestr)
        return suppression_filter

    def handle_result(self, result, **kwargs):
        previous_state = self.get_state(result.id)
        for handler_name, handler in self._handlers.iteritems():
            try:
                # We do suppression AFTER filters, so we have to
                # pass Reactor to the handler to do that for us
                handler._process(result, previous_state, self.is_suppressed)
            except Exception:
                logutil.log_exception("Unhandled %s handler "
                                      "exception:" % (handler_name,), logger)
                continue
        try:
            self.save_state(result.id, result, previous_state)
        except OutOfDateState:
            pass

    def run(self, handler_config_path, **kwargs):
        """ This will run in a tight loop. It is expected that the subclass's
        get_result() method will introduce a delay if the results queue is
        empty.
        """
        self._load_handlers(handler_config_path, **kwargs)
        while True:
            result = self.get_result(**kwargs)
            if not result:
                logger.debug('Result queue empty.')
                continue
            self.handle_result(result, **kwargs)
            result.delete()



import logging

logger = logging.getLogger(__name__)

import time

from boto.exception import SDBResponseError

from nymms.scheduler.lock.SchedulerLock import SchedulerLock


class SDBLock(SchedulerLock):
    def __init__(self, duration, conn, domain_name,
                 lock_name="scheduler_lock"):
        super(SDBLock, self).__init__(duration, lock_name)
        self.conn = conn
        self.domain_name = domain_name
        self.domain = None
        self.lock = None

    def setup_domain(self):
        if self.domain:
            return
        logger.debug("Setting up lock domain %s", self.domain_name)
        self.domain = self.conn.create_domain(self.domain_name)

    def acquire(self):
        logger.debug("Attempting to acquire lock %s:%s", self.domain_name,
                     self.lock_name)
        self.setup_domain()
        now = int(time.time())
        existing_lock = self.domain.get_item(self.lock_name,
                                              consistent_read=True)
        lock_body = {'expiry': now + self.duration,
                     'timestamp': now,
                     'owner': self.id}
        expected_value = ['timestamp', False]
        if existing_lock:
            logger.debug("Existing lock found: %s", existing_lock)
            existing_ts = existing_lock['timestamp']
            if not existing_lock['owner'] == self.id:
                if not self.lock_expired(existing_lock['expiry'], now):
                    logger.debug("Lock still valid, not taking over.")
                    return False
                else:
                    logger.info("Lock expired, attempting takeover.")
            else:
                logger.info("I already own the lock, updating.")
            expected_value = ['timestamp', existing_ts]

        try:
            self.domain.put_attributes(self.lock_name, lock_body,
                                        replace=bool(existing_lock),
                                        expected_value=expected_value)
            self.lock = lock_body
            logger.debug("Acquired lock %s:%s", self.domain_name,
                         self.lock_name)
            return True
        except SDBResponseError as e:
            if e.status == 409:
                logger.debug('Looks like someone else has acquired the lock.')
                return False
            raise
        return False



import unittest
import time
import os

os.environ['PATH'] += ":/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin"
os.environ['PATH'] += ":/usr/local/sbin"

from nymms.probe.Probe import Probe, TIMEOUT_OUTPUT
from nymms import results, resources
from nymms.tasks import Task

result_codes = [
    results.CRITICAL,
    results.WARNING,
    results.CRITICAL,
    results.WARNING,
    results.WARNING,
    results.OK,
    results.OK,
    results.UNKNOWN,
    results.OK,
    results.OK
]

true_output = "Good output"
true_command = resources.Command('true_command', 'echo ' + true_output)
fail_command = resources.Command('fail_command', 'false')
sleep_command = resources.Command('sleep_command', 'sleep {{sleep_time}}')

true_monitor = resources.Monitor('true_monitor', command=true_command)
fail_monitor = resources.Monitor('fail_monitor', command=fail_command)
sleep_monitor = resources.Monitor('sleep_monitor', command=sleep_command)


class DummyStateBackend(object):
    def __init__(self):
        self.states = [
            None,
            {'last_update': 1, 'last_state_change': 0,
                'state': results.CRITICAL, 'state_type': results.SOFT},
            {'last_update': 2, 'last_state_change': 0,
                'state': results.WARNING, 'state_type': results.SOFT},
            {'last_update': 3, 'last_state_change': 3,
                'state': results.CRITICAL, 'state_type': results.HARD},
            {'last_update': 4, 'last_state_change': 4,
                'state': results.WARNING, 'state_type': results.HARD},
            {'last_update': 5, 'last_state_change': 4,
                'state': results.WARNING, 'state_type': results.HARD},
            {'last_update': 6, 'last_state_change': 6,
                'state': results.OK, 'state_type': results.HARD},
            {'last_update': 7, 'last_state_change': 6,
                'state': results.OK, 'state_type': results.HARD},
            {'last_update': 8, 'last_state_change': 8,
                'state': results.UNKNOWN, 'state_type': results.SOFT},
            {'last_update': 9, 'last_state_change': 9,
                'state': results.OK, 'state_type': results.SOFT},
            {'last_update': 10, 'last_state_change': 9,
                'state': results.OK, 'state_type': results.HARD}
        ]
        self.state_iter = iter(self.states)

    def get_state(self, task_id):
        state_item = next(self.state_iter)
        state = None
        if state_item:
            state_item['id'] = task_id
            state = results.StateRecord.deserialize(state_item)
            state.validate()
        return state


class DummyProbe(Probe):
    def __init__(self):
        self.task = Task('test:task',
                         context={
                             'monitor': {
                                 'name': 'true_monitor'
                             }
                         })
        self.results_iter = iter(result_codes)

    def get_task(self, **kwargs):
        return self.task

    def resubmit_task(self, task, delay, **kwargs):
        self.task.increment_attempt()

    def submit_result(self, result, **kwargs):
        if result.state_type == results.HARD:
            self.task.attempt = 0
        return result

    def execute_task(self, task, timeout, **kwargs):
        result = results.Result(task.id, timestamp=task.created,
                                task_context=task.context)
        result.state = next(self.results_iter)
        result.output = 'Some output here.'
        return result


class TestStateChange(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.state_backend = DummyStateBackend()
        cls.probe = DummyProbe()
        cls.probe._state_backend = cls.state_backend

    def test_state_change(self):
        # tests that our logic follows the nagios logic here
        # http://nagios.sourceforge.net/docs/3_0/statetypes.html
        # We take our example state changes from the table at the bottom
        t = self.probe.get_task()
        for i, code in enumerate(result_codes):
            r = self.probe.handle_task(t, monitor_timeout=30,
                                       max_retries=2)
            expected = self.state_backend.states[i + 1]
            print "[%d] Result STATE/TYPE: %s/%s" % (i, r.state, r.state_type)
            print "[%d] Expected STATE/TYPE: %s/%s" % (i, expected['state'],
                                                       expected['state_type'])
            self.assertEqual(r.state, expected['state'])
            self.assertEqual(r.state_type, expected['state_type'])
            self.probe.submit_result(r)

    def test_expiration(self):
        expiration = 30
        now = time.time()
        t = self.probe.get_task()
        t.created = now
        self.assertFalse(self.probe.expire_task(t, expiration))
        t.created = now - (expiration + 5)
        self.assertTrue(self.probe.expire_task(t, expiration))


class TestExecuteTask(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.probe = Probe()
        cls.true_task = Task('test:true_monitor',
                             context={
                                 'monitor': {
                                     'name': 'true_monitor',
                                 }
                             })
        cls.fail_task = Task('test:fail_monitor',
                             context={
                                 'monitor': {
                                     'name': 'fail_monitor',
                                 }
                             })
        cls.timeout_task = Task('test:timeout_monitor',
                             context={
                                 'monitor': {
                                     'name': 'sleep_monitor',
                                 },
                                 'sleep_time': 2,
                             })
        cls.probe._private_context = {}

    def test_successful_execute_task(self):
        result = self.probe.execute_task(self.true_task, 30)
        print result.output
        self.assertEqual(result.state, results.OK)
        self.assertEqual(result.output.strip(), true_output)

    def test_failed_execute_task(self):
        result = self.probe.execute_task(self.fail_task, 30)
        self.assertEqual(result.state, results.WARNING)
        self.assertEqual(result.output.strip(), '')

    def test_timeout_execute_task(self):
        timeout = 1
        result = self.probe.execute_task(self.timeout_task, timeout)
        self.assertEqual(result.state, results.UNKNOWN)
        self.assertEqual(result.output.strip(), TIMEOUT_OUTPUT % timeout)



import time
import sys
import re
import argparse
from nymms.utils.cli import NymmsCommandArgs
import logger



class SuppressCommandArgs(NymmsCommandArgs):
    def __init__(self, *args, **kwargs):
        super(SuppressCommandArgs, self).__init__(*args, **kwargs)
        self.add_argument('-r', '--region', dest='region',
                help='Override config AWS region to connect to')
        self.add_argument('-d', '--domain', dest='domain',
                help='Override config AWS SDB Domain to use')
        self.region = None
        self.domain = None
        self.reference_time = int(time.time())
        self.values = None

    def parse_args(self):
        self.values = super(SuppressCommandArgs, self).parse_args()
        return self.values

    def load_config(self):
        # load nymms.config here to avoid the error: 
        # No handlers could be found for logger "nymms.config.config"
        from nymms.config import config

        # prefer CLI values if we have them
        if self.values.region:
            self.region = self.values.region
        if self.values.domain:
            self.domain = self.values.domain

        # fallback to the config file if we have to
        if not self.domain or not self.region:
            try:
                config.load_config(self.values.config)
            except IOError:
                logger.error("Please specify --region and --domain")
                exit(-1)

            if not self.region:
                self.region = config.settings['region']
            if not self.domain:
                self.domain = config.settings['suppress']['domain']

    def parse_time(self, expires):
        """Parses a time in YYYYMMDDHHMMSS or +XXXX[smhd] and returns
        epoch time

        if expires == 0, returns None"""
        if expires == '0':
            return None

        if expires[0] == '+' or expires[0] == '-':
            last_char = expires[len(expires) - 1]
            user_value = expires[0:(len(expires) - 1)]
            if last_char == 's':
                epoch = self.reference_time + int(user_value)
            elif last_char == 'm':
                epoch = self.reference_time + (int(user_value) * 60)
            elif last_char == 'h':
                epoch = self.reference_time + (int(user_value) * 60 * 60)
            elif last_char == 'd':
                epoch = self.reference_time + (int(user_value) * 60 * 60 * 24)
            else:
                sys.stderr.write("Invalid time format: %s.  " +
                        "Missing s/m/h/d qualifier\n", expires)
                exit(-1)
        else:
            epoch = int(time.strftime("%s",
                time.strptime(expires, "%Y%m%d%H%M%S")))

        return epoch



from jinja2 import Undefined


class SimpleUndefined(Undefined):
    """ A version of undefined that doesn't freak out when a context
    variable is missing and gives non-verbose help.  Unfortunately it's all
    but impossible with jinja to return the name of a dictionary that is
    missing a key.  We'll make do for now.

    This is a weak hack - I really need to find a way to provide more
    'context' about what is missing in the context.
    """
    __slots__ = ()

    def __unicode__(self):
        return u'{{MISSING_CONTEXT}}'
    def __getattr__(self, attr):
        return u'{{MISSING_CONTEXT}}'
    def __getitem__(self, item):
        return u'{{MISSING_CONTEXT}}'



import logging

logger = logging.getLogger(__name__)

import uuid
import time


class SchedulerLock(object):
    def __init__(self, duration, lock_name="scheduler_lock"):
        self.id = self.get_instance_id()
        self.duration = duration
        self.lock_name = lock_name
        logger.debug("%s:%s initialized with %s duration.",
                     self.__class__.__name__, self.id, duration)

    def get_instance_id(self):
        """ Can be overridden, but a random UUID at launch is probably good
        enough.
        """
        return uuid.uuid4().hex

    def lock_expired(self, expiry, now):
        """ Returns True if the lock is expired, False otherwise.  """
        if not expiry or int(now) > int(expiry):
            return True
        return False

    def acquire(self):
        """ Should be overridden and return True or False depending on whether
        it got the lock or not.
        """
        raise NotImplemented


class NoOpLock(SchedulerLock):
    def __init__(self):
        logger.warning("!!! Using NoOpLock")
        logger.warning("!!! Do not do this if you are planning to run more ")
        logger.warning("!!! than one scheduler.""")

    def acquire(self):
        return True



import unittest

from nymms.reactor.Reactor import Reactor
from nymms.reactor.handlers.Handler import Handler


enabled_config = {'handler_class': 'nymms.reactor.handlers.Handler.Handler',
                  'enabled': True}
disabled_config = {'handler_class': 'nymms.reactor.handlers.Handler.Handler',
                   'enabled': False}


class TestReactor(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.reactor = Reactor()

    def test_load_enabled_handler(self):
        handler = self.reactor._load_handler('dummy_handler', enabled_config)
        self.assertIsInstance(handler, Handler)

    def test_load_disabled_handler(self):
        handler = self.reactor._load_handler('dummy_handler', disabled_config)
        self.assertIs(handler, None)



import logging
import traceback
import time
from nymms.suppress.suppress import SuppressFilterBackend, ReactorSuppress

logger = logging.getLogger(__name__)


class SDBSuppressFilterBackend(SuppressFilterBackend):
    def __init__(self, conn, timeout=60, domain_name='nymms_suppress'):
        self._conn = conn
        self._domain_name = domain_name
        self.domain = None
        super(SDBSuppressFilterBackend, self).__init__(timeout)
        logger.debug("%s initialized.", self.__class__.__name__)

    def _setup_domain(self):
        if self.domain:
            return
        logger.debug("setting up reactor suppression domain %s",
                self._domain_name)
        self.domain = self._conn.create_domain(self._domain_name)

    def add_suppression(self, suppress):
        """Adds a suppression filter to the SDB store
        """
        self._setup_domain()
        self.domain.put_attributes(suppress.rowkey, suppress.dict())
        logger.debug("Added %s to %s", suppress.rowkey, self._domain_name)
        return suppress.rowkey

    def get_suppressions(self, expire, active=True):
        """Returns a list of suppression filters which were active between
        start and end

        expire = expoch time
        active = True/False to limit to only filters flagged 'active' = 'True'
        """
        self._setup_domain()
        if expire:
            query = "select * from `%s` where `expires` >= '%s'" % (
                    self._domain_name, expire)
        else:
            query = "select * from `%s` where `expires` > '0'" % (
                    self._domain_name,)

        if active:
            query += " and `active` = 'True'"
        query += " order by expires"

        suppressors = []
        for item in self.domain.select(query):
            suppressors.append(ReactorSuppress(item))

        return suppressors

    def deactivate_suppression(self, rowkey):
        """Deactivates a single suppression filter"""
        self._setup_domain()
        self._conn.put_attributes(self._domain_name, rowkey,
                {'active': int(time.time())})



import argparse
import logging
import sys
import time

logger = logging.getLogger(__name__)

from nymms.utils import logutil


def setup_logging(verbose=False):
    level = logutil.INFO
    if verbose:
        level = logutil.DEBUG
    if not verbose > 1:
        logutil.quiet_boto_logging()
    return logutil.setup_root_logger(stdout=level)


def parse_time(time_string, reference_time=int(time.time())):
    """Parses a time in YYYYMMDDHHMMSS or +XXXX[smhd] and returns
    epoch time

    reference_time should be the epoch time used for calculating
    the time when using +XXXX[smhd]

    if time_string == 0, returns None"""
    if time_string == '0':
        return None

    if time_string[0] == '+' or time_string[0] == '-':
        last_char = time_string[len(time_string) - 1]
        user_value = time_string[0:(len(time_string) - 1)]
        if last_char == 's':
            epoch = reference_time + int(user_value)
        elif last_char == 'm':
            epoch = reference_time + (int(user_value) * 60)
        elif last_char == 'h':
            epoch = reference_time + (int(user_value) * 60 * 60)
        elif last_char == 'd':
            epoch = reference_time + (int(user_value) * 60 * 60 * 24)
        else:
            sys.stderr.write("Invalid time format: %s.  " +
                    "Missing s/m/h/d qualifier\n", time_string)
            exit(-1)
    else:
        epoch = int(time.strftime("%s",
            time.strptime(time_string, "%Y%m%d%H%M%S")))

    return epoch


class NymmsCommandArgs(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        super(NymmsCommandArgs, self).__init__(*args, **kwargs)
        self.add_argument('-v', '--verbose', action='count', default=0)
        self.add_argument('-c', '--config', default='/etc/nymms/config.yaml')



import unittest

from nymms import utils


class DummyClass(object):
    pass


class TestBase(unittest.TestCase):
    def test_load_class_from_string(self):
        from logging import Logger
        cls = utils.load_object_from_string('logging.Logger')
        self.assertIs(cls, Logger)



import logging
import time

import nymms
from nymms.daemon import NymmsDaemon
from nymms.resources import Node
from nymms.tasks import Task
from nymms.scheduler.lock.SchedulerLock import NoOpLock

logger = logging.getLogger(__name__)


class Scheduler(NymmsDaemon):
    task_id_template = "{node[name]}:{monitor[name]}"

    def __init__(self, node_backend, lock=None):
        self._node_backend = node_backend
        if not lock:
            lock = NoOpLock()

        self._lock = lock
        super(Scheduler, self).__init__()

    def get_tasks(self):
        tasks = {}
        self._node_backend._load_nodes()
        nodes = Node.registry
        for node_name, node in nodes.iteritems():
            tasks[node_name] = node.monitors
        return tasks

    def submit_task(self, task, **kwargs):
        raise NotImplementedError

    def run(self, **kwargs):
        interval = kwargs.get('interval')
        while True:
            start = time.time()
            if self._lock.acquire():
                self.run_once(**kwargs)
                run_time = time.time() - start
                logger.info("Scheduler iteration took %d seconds.", run_time)
                sleep_time = interval - max(run_time, 0)
                logger.info("Scheduler sleeping for %d seconds.", sleep_time)
            else:
                # Only sleep for 10 seconds before checking the lock again
                # when we don't acquire the lock. Allows for faster takeover.
                sleep_time = 10
                logger.info("Failed to acquire lock, sleeping for %d seconds.",
                            sleep_time)
            time.sleep(sleep_time)

    def run_once(self, **kwargs):
        tasks = self.get_tasks()
        # This is done to make sure we submit one task per node until we've
        # submitted all the tasks.  This helps ensure we don't hammer a
        # single node with monitoring tasks
        while True:
            working_index = tasks.keys()
            if not working_index:
                break
            for node in working_index:
                try:
                    task_context = tasks[node].pop()
                    task_id = self.task_id_template.format(**task_context)
                    task = Task(task_id, context=task_context)
                except IndexError:
                    del(tasks[node])
                    continue
                self.submit_task(task, **kwargs)

